; *************************************
; *
; * RS 232 handling
; *
; *************************************

; -------------------------------------
; getRS - get byte from rs232
;
; affects: A, X, Y
; return: char in Y
;         c=0 on valid data
;
; uses kernal or up9600 routines
; -------------------------------------
getRS   lda Baud    ; up9600 active?
        cmp #$0c
        bne RS_kern

	; -- read RS232 from up9600 --
        jsr rsin    ; get char
        tay         ; char to Y
        rts

RS_kern ; -- read RS232 from kernal --
	ldx #$01    ; 1 = lfn for rs232
        jsr CHKIN
	jsr GETIN   ; get char
        tay         ; save char in Y
	jsr READST
	pha	    ; save ST
	jsr CLRCHN
	pla	    ; restore ST
	clc
	adc #$ff    ; c=1 if ST!=0
	rts

; -------------------------------------
; XgetRS - get byte from RS232 with 
;          timeout
;
; affects: A, X, Y
; return: data in Y
;         c=0 -> data ok
;         c=1 -> timeout
; -------------------------------------
XgetRS  lda ClockM    ; get clock
        sta Xtime
XgetRS1 jsr getRS     ; get char
        bcc XgetRSe   ; valid?
        lda ClockM    ; timeout..
        sbc Xtime     ; ..ca. 8s..
        cmp #$02      ; ..over?
        bcc XgetRS1   ; no -> wait        
XgetRSe rts

; -------------------------------------
; putRS - send char to RS232
;
; params: char in A
; affects: A, X
;
; uses kernal or up9600 routines
; -------------------------------------
putRS   ldx Baud    ; up9600 active?
        cpx #$0c
        bne PRS1    ; to Kernal out

	; -- send char to up9600 --
        jmp rsout

PRS1    ; -- send char to kernal --
	pha	    ; save char
	ldx #$01    ; 1 = lfn for rs232
        jsr CHKOUT
	pla	    ; restore char
	jsr CHROUT
	jsr CLRCHN
	rts

; -------------------------------------
; SendStr - send string to rs232
;
; params: string ptr in X (lo), Y (hi)
; affects: A, X, Y
; uses: zVector
;
; string: chars, terminated by $00
; the string must be smaller than 255 chrs
; -------------------------------------
SendStr stx zVector
	sty zVector+1
	ldy #$00
L3      lda (zVector),y
        beq L4      ; $00 terminates string
        jsr putRS        
        iny
        jmp L3

L4      rts

; -------------------------------------
; Open rs232 channel
;
; affects: A, X, Y
; uses: xVector
;
; initialises rs232 channel via 
; KERNAL or up9600 routines
; -------------------------------------

; first byte of filename in Baud.
; $06 = 300 baud
; $08 = 1200 baud
; $0a = 2400 baud
; $0c = 9600 baud
; $0e = IEC bus

InitCh  lda Baud
        cmp #$0c
        bne ICh2
        
	; --- open up9600 ---
        jsr install
        bcc ICh1    ; error?
        ldx #<SError
        ldy #>SError
        jsr CPrnStrNL
ICh1    rts

SError  .asc "up9600 error."
        .byt $00        

	; --- open rs232 via KERNAL ---
ICh2    cmp #$0e    ; IEC bus?
	beq ICh3
        ; paremeters for rs232
	lda #$01    ; channel 1
        ldx #$02    ; 2 = rs232
        ldy #$ff    ; no command
        jsr SETLFS
	; prepare baud index
	sec
        lda Baud    ; $06, $08, $0a
	sbc #$06    ; -> $00, $02, $04
	asl	    ; -> $00, $04, $08
	; select NTSC/PAL
        ldx $02a6   ; 0=NTSC 1=PAL
	bne ICh4
	adc #12	    ; switch to NTSC tab
ICh4	; prepare filename pointer
	adc #<BaudTab
	tax	    ; lo byte file name
	lda #$00
	adc #>BaudTab
	tay	    ; hi byte file name
	; open serial port
	lda #$04    ; file name length
        jsr SETNAM
        jsr OPEN
        rts

BaudTab ; --- PAL ---
	; 300 baud, 8n1
	; timer 1542, original val
	.byt $00, $00
	.word $0606 
	; 1200 baud, 8n1
	; kernal val 311, 1196 measured
	; new val 306, 1207 measured
	.byt $00, $00
	.word $0132 
	; 2400 baud, 8n1 (host at 8n2)
	; kernal val 105
	; new val 102, 2424 measured
	.byt $00, $00
	.word $0066 
	; --- NTSC ---
	; 300 baud, 8n1, original val
	.byt $06, $00
	.word $0000
	; 1200 baud, 8n1, original val
	.byt $08, $00
	.word $0000
	; 2400 baud, 8n1, original val
	.byt $0a, $00
	.word $0000

        ; --- open IEC bus ---
ICh3	lda #$01    ;channel 1
        ldx IECdevice
        ldy #$02    ;secondary addr
        jsr SETLFS
	lda IECstrLen
	ldx #<IECstring
	ldy #>IECstring
        jsr SETNAM
        jsr OPEN
        rts


; -------------------------------------
; close rs232 channel
;
; affects: A, X, Y
;
; works on KERNAL channel or on up9600
; -------------------------------------
CloseCh lda Baud
        cmp #$0c
        bne CCK

; --- close up9600 ---
        jmp uninstall

; --- close KERNAL channel
CCK     lda #$01    ; close lfn 1
        jmp CLOSE

; -------------------------------------
; XUPoff - disables up9600 if active 
; 
; affects: A, X
; -------------------------------------
XUPoff  lda Baud    ; up9600 active?
        cmp #$0c
        bne XUPoff1
        jsr disable
XUPoff1 rts

; -------------------------------------
; XUPon - enables up9600 if active 
; 
; affects: A, X, Y
; -------------------------------------
XUPon   lda Baud      ; up9600 active?
        cmp #$0c
        bne XUPon2
        jsr WaitMs    ; wait 2 ms..
        jsr WaitMs    ; ..for ICE bus
        jsr enable
XUPon2  rts

; vim: set syntax=asm6502 :

